import pandas as pd
from pathlib import Path
from ipc.downloader.geo_microarray.gpl_auto_annotation import ontology_functions


def init_mapper():
    """
    This is a function which would create a dictionary mapping ontology function defined with the known column names.

    Returns:
        dict
    """
    df=pd.read_csv(str(Path(__file__).parent) +"/Gene_ontology_dictionary.csv")
    mapper=dict()
    

    for row in range(df.shape[0]):
        ontology = df.iloc[row,1]
        col_name = df.iloc[row,0]
        supported_ontology={"assoc_gene":ontology_functions.is_assoc_gene, "ensembl":ontology_functions.is_ensembl,"entrez":ontology_functions.is_entrez,"genbank":ontology_functions.is_genbank,"genbank_gblist":ontology_functions.is_genbank_gblist,"gene_ass":ontology_functions.is_gene_ass,"symbol":ontology_functions.is_symbol}
        if ontology in supported_ontology:
            add_to(mapper,col_name,supported_ontology[ontology])
    return mapper

def add_to(arr,key,value):
    """
    This is a helper function which would add key value pairs to a dictionary.

    Args:
        arr (dict): dictionary in which the key, value pair is to be added.
        key (str): column name which needs to be mapped.
        value (function pointer): pointer to the function that column needs to be mapped to.
    """
    if key in arr:
        arr[key].append(value)
    else:
        arr[key]=[value]

    
def rev_map(x):
    """
    This is a function that would map the functions pointers with their ontology names.

    Args:
        x (function pointer): pointer to the function

    Returns:
        ontology name (str): Name of the ontology that x function refers to.
    """
    supported_ontology={ontology_functions.is_assoc_gene:"assoc_gene", ontology_functions.is_ensembl:"ensembl",ontology_functions.is_entrez:"entrez",ontology_functions.is_genbank:"genbank",ontology_functions.is_genbank_gblist:"genbank_gblist",ontology_functions.is_gene_ass:"gene_ass",ontology_functions.is_symbol:"symbol"}
    return supported_ontology[x]  